@@ -9,10 +9,10 @@
 
 #pragma once
 
-#include <QObject>
-
 #include "kleo_export.h"
 
+#include <QObject>
+
 #include <gpgme++/global.h>
 
 #include <memory>
@@ -41,21 +41,33 @@ namespace Kleo
 
 class FileSystemWatcher;
 class KeyGroup;
-enum class KeyUsage : char;
+class KeyGroupConfig;
+
+class KeyCacheAutoRefreshSuspension;
 
 class KLEO_EXPORT KeyCache : public QObject
 {
     Q_OBJECT
+
 protected:
     explicit KeyCache();
+
 public:
+    enum class KeyUsage {
+        AnyUsage,
+        Sign,
+        Encrypt,
+        Certify,
+        Authenticate,
+    };
+
     static std::shared_ptr<const KeyCache> instance();
     static std::shared_ptr<KeyCache> mutableInstance();
 
     ~KeyCache() override;
 
     void setGroupsEnabled(bool enabled);
-    void setGroupsConfig(const QString &filename);
+    void setGroupConfig(const std::shared_ptr<KeyGroupConfig> &groupConfig);
 
     void insert(const GpgME::Key &key);
     void insert(const std::vector<GpgME::Key> &keys);
@@ -75,12 +87,15 @@ public:
     void setRefreshInterval(int hours);
     int refreshInterval() const;
 
+    std::shared_ptr<KeyCacheAutoRefreshSuspension> suspendAutoRefresh();
+
     void enableRemarks(bool enable);
     bool remarksEnabled() const;
 
     const std::vector<GpgME::Key> &keys() const;
     std::vector<GpgME::Key> secretKeys() const;
 
+    KeyGroup group(const QString &id) const;
     std::vector<KeyGroup> groups() const;
     std::vector<KeyGroup> configurableGroups() const;
     void saveConfigurableGroups(const std::vector<KeyGroup> &groups);
@@ -88,6 +103,8 @@ public:
     const GpgME::Key &findByFingerprint(const char *fpr) const;
     const GpgME::Key &findByFingerprint(const std::string &fpr) const;
 
+    std::vector<GpgME::Key> findByFingerprint(const std::vector<std::string> &fprs) const;
+
     std::vector<GpgME::Key> findByEMailAddress(const char *email) const;
     std::vector<GpgME::Key> findByEMailAddress(const std::string &email) const;
 
@@ -146,19 +163,20 @@ public:
     std::vector<GpgME::Key> getGroupKeys(const QString &groupName) const;
 
     enum Option {
-        NoOption = 0,
+        // clang-format off
+        NoOption        = 0,
         RecursiveSearch = 1,
-        IncludeSubject = 2
+        IncludeSubject  = 2,
+        // clang-format on
     };
     Q_DECLARE_FLAGS(Options, Option)
 
     std::vector<GpgME::Key> findSubjects(const GpgME::Key &key, Options option = RecursiveSearch) const;
     std::vector<GpgME::Key> findSubjects(const std::vector<GpgME::Key> &keys, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findSubjects(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
+    std::vector<GpgME::Key>
+    findSubjects(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
 
     std::vector<GpgME::Key> findIssuers(const GpgME::Key &key, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findIssuers(const std::vector<GpgME::Key> &keys, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findIssuers(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
 
     /** Check if at least one keylisting was finished. */
     bool initialized() const;
@@ -181,7 +199,7 @@ public Q_SLOTS:
     void cancelKeyListing();
 
 Q_SIGNALS:
-    //void changed( const GpgME::Key & key );
+    // void changed( const GpgME::Key & key );
     void aboutToRemove(const GpgME::Key &key);
     void added(const GpgME::Key &key);
     void keyListingDone(const GpgME::KeyListResult &result);
@@ -200,4 +218,3 @@ private:
 }
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Kleo::KeyCache::Options)
-
