@@ -2,58 +2,26 @@
     keyrequester.h
 
     This file is part of libkleopatra, the KDE keymanagement library
-    Copyright (c) 2004 Klarälvdalens Datakonsult AB
-
-    Libkleopatra is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License as
-    published by the Free Software Foundation; either version 2 of the
-    License, or (at your option) any later version.
-
-    Libkleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
 
     Based on kpgpui.h
-    Copyright (C) 2001,2002 the KPGP authors
+    SPDX-FileCopyrightText: 2001, 2002 the KPGP authors
     See file libkdenetwork/AUTHORS.kpgp for details
 
     This file is part of KPGP, the KDE PGP/GnuPG support library.
 
-    KPGP is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software Foundation,
-    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+    SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifndef __KLEO_UI_KEYREQUESTER_H__
-#define __KLEO_UI_KEYREQUESTER_H__
+#pragma once
 
 #include "kleo_export.h"
 
-#include <qgpgme/protocol.h>
+#include <QGpgME/Protocol>
 
-#include <QWidget>
 #include <QLabel>
+#include <QStringList>
+#include <QWidget>
 
 #include <vector>
 
@@ -63,10 +31,8 @@ class Key;
 class KeyListResult;
 }
 
-class QStringList;
 class QString;
 class QPushButton;
-class QLabel;
 
 namespace Kleo
 {
@@ -76,11 +42,10 @@ class KLEO_EXPORT KeyRequester : public QWidget
 {
     Q_OBJECT
 public:
-    explicit KeyRequester(unsigned int allowedKeys, bool multipleKeys = false,
-                          QWidget *parent = nullptr);
+    explicit KeyRequester(unsigned int allowedKeys, bool multipleKeys = false, QWidget *parent = nullptr);
     // Constructor for Qt Designer
     explicit KeyRequester(QWidget *parent = nullptr);
-    ~KeyRequester();
+    ~KeyRequester() override;
 
     const GpgME::Key &key() const;
     /** Preferred method to set a key for
@@ -158,6 +123,7 @@ private:
 private:
     class Private;
     Private *const d;
+
 protected:
     virtual void virtual_hook(int, void *);
 };
@@ -180,13 +146,14 @@ public:
      * Constructor for Qt designer
      */
     explicit EncryptionKeyRequester(QWidget *parent);
-    ~EncryptionKeyRequester();
+    ~EncryptionKeyRequester() override;
 
     void setAllowedKeys(unsigned int proto, bool onlyTrusted = true, bool onlyValid = true);
 
 private:
     class Private;
     Private *const d;
+
 protected:
     void virtual_hook(int, void *) override;
 };
@@ -209,12 +176,13 @@ public:
     explicit SigningKeyRequester(bool multipleKeys = false,
                                  unsigned int proto = AllProtocols,
                                  QWidget *parent = nullptr,
-                                 bool onlyTrusted = true, bool onlyValid = true);
+                                 bool onlyTrusted = true,
+                                 bool onlyValid = true);
     /**
      * Constructor for Qt designer
      */
     explicit SigningKeyRequester(QWidget *parent);
-    ~SigningKeyRequester();
+    ~SigningKeyRequester() override;
 
     /*
      * Those parameters affect the parameters given to the key selection dialog.
@@ -227,10 +195,9 @@ public:
 private:
     class Private;
     Private *const d;
+
 protected:
     void virtual_hook(int, void *) override;
 };
 
 }
-
-#endif // __KLEO_UI_KEYREQUESTER_H__
