@@ -7,15 +7,19 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "filesystemwatcher.h"
-#include "kleo/stl_util.h"
+
+#include <libkleo/stl_util.h>
 
 #include <libkleo_debug.h>
 
+#include <QDir>
 #include <QFileSystemWatcher>
+#include <QRegExp>
 #include <QString>
 #include <QTimer>
-#include <QDir>
 
 #include <set>
 
@@ -24,6 +28,7 @@ using namespace Kleo;
 class FileSystemWatcher::Private
 {
     FileSystemWatcher *const q;
+
 public:
     explicit Private(FileSystemWatcher *qq, const QStringList &paths = QStringList());
     ~Private()
@@ -47,20 +52,23 @@ public:
 };
 
 FileSystemWatcher::Private::Private(FileSystemWatcher *qq, const QStringList &paths)
-    : q(qq),
-      m_watcher(nullptr),
-      m_paths(paths)
+    : q(qq)
+    , m_watcher(nullptr)
+    , m_paths(paths)
 {
     m_timer.setSingleShot(true);
-    connect(&m_timer, &QTimer::timeout, q, [this]() { onTimeout(); });
+    connect(&m_timer, &QTimer::timeout, q, [this]() {
+        onTimeout();
+    });
 }
 
 static bool is_matching(const QString &file, const QStringList &list)
 {
-    for (const QString &entry : list)
+    for (const QString &entry : list) {
         if (QRegExp(entry, Qt::CaseInsensitive, QRegExp::Wildcard).exactMatch(file)) {
             return true;
         }
+    }
     return false;
 }
 
@@ -72,7 +80,7 @@ static bool is_blacklisted(const QString &file, const QStringList &blacklist)
 static bool is_whitelisted(const QString &file, const QStringList &whitelist)
 {
     if (whitelist.empty()) {
-        return true;    // special case
+        return true; // special case
     }
     return is_matching(file, whitelist);
 }
@@ -87,7 +95,11 @@ void FileSystemWatcher::Private::onFileChanged(const QString &path)
         return;
     }
     qCDebug(LIBKLEO_LOG) << path;
-    m_seenPaths.insert(path);
+    if (fi.exists()) {
+        m_seenPaths.insert(path);
+    } else {
+        m_seenPaths.erase(path);
+    }
     m_cachedFiles.insert(path);
     handleTimer();
 }
@@ -96,23 +108,20 @@ static QStringList list_dir_absolute(const QString &path, const QStringList &bla
 {
     QDir dir(path);
     QStringList entries = dir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot);
-    QStringList::iterator end =
-        std::remove_if(entries.begin(), entries.end(),
-                       [&blacklist](const QString &entry) {
-                           return is_blacklisted(entry, blacklist);
-                       });
-    if (!whitelist.empty())
-        end = std::remove_if(entries.begin(), end,
-                             [&whitelist](const QString &entry) {
-                                 return !is_whitelisted(entry, whitelist);
-                             });
+    QStringList::iterator end = std::remove_if(entries.begin(), entries.end(), [&blacklist](const QString &entry) {
+        return is_blacklisted(entry, blacklist);
+    });
+    if (!whitelist.empty()) {
+        end = std::remove_if(entries.begin(), end, [&whitelist](const QString &entry) {
+            return !is_whitelisted(entry, whitelist);
+        });
+    }
     entries.erase(end, entries.end());
     std::sort(entries.begin(), entries.end());
 
-    std::transform(entries.begin(), entries.end(), entries.begin(),
-                   [&dir](const QString &entry) {
-                       return dir.absoluteFilePath(entry);
-                   });
+    std::transform(entries.begin(), entries.end(), entries.begin(), [&dir](const QString &entry) {
+        return dir.absoluteFilePath(entry);
+    });
 
     return entries;
 }
@@ -120,9 +129,7 @@ static QStringList list_dir_absolute(const QString &path, const QStringList &bla
 static QStringList find_new_files(const QStringList &current, const std::set<QString> &seen)
 {
     QStringList result;
-    std::set_difference(current.begin(), current.end(),
-                        seen.begin(), seen.end(),
-                        std::back_inserter(result));
+    std::set_difference(current.begin(), current.end(), seen.begin(), seen.end(), std::back_inserter(result));
     return result;
 }
 
@@ -145,7 +152,8 @@ void FileSystemWatcher::Private::onDirectoryChanged(const QString &path)
 
 void FileSystemWatcher::Private::onTimeout()
 {
-    std::set<QString> dirs, files;
+    std::set<QString> dirs;
+    std::set<QString> files;
 
     dirs.swap(m_cachedDirectories);
     files.swap(m_cachedFiles);
@@ -178,18 +186,24 @@ void FileSystemWatcher::Private::connectWatcher()
     if (!m_watcher) {
         return;
     }
-    connect(m_watcher, &QFileSystemWatcher::directoryChanged, q, [this](const QString &str) { onDirectoryChanged(str); });
-    connect(m_watcher, &QFileSystemWatcher::fileChanged, q, [this](const QString &str) { onFileChanged(str); });
+    connect(m_watcher, &QFileSystemWatcher::directoryChanged, q, [this](const QString &str) {
+        onDirectoryChanged(str);
+    });
+    connect(m_watcher, &QFileSystemWatcher::fileChanged, q, [this](const QString &str) {
+        onFileChanged(str);
+    });
 }
 
 FileSystemWatcher::FileSystemWatcher(QObject *p)
-    : QObject(p), d(new Private(this))
+    : QObject(p)
+    , d(new Private(this))
 {
     setEnabled(true);
 }
 
 FileSystemWatcher::FileSystemWatcher(const QStringList &paths, QObject *p)
-    : QObject(p), d(new Private(this, paths))
+    : QObject(p)
+    , d(new Private(this, paths))
 {
     setEnabled(true);
 }
@@ -237,11 +251,15 @@ void FileSystemWatcher::blacklistFiles(const QStringList &paths)
 {
     d->m_blacklist += paths;
     QStringList blacklisted;
-    d->m_paths.erase(kdtools::separate_if(d->m_paths.begin(), d->m_paths.end(),
-                                          std::back_inserter(blacklisted), d->m_paths.begin(),
+    d->m_paths.erase(kdtools::separate_if(d->m_paths.begin(),
+                                          d->m_paths.end(),
+                                          std::back_inserter(blacklisted),
+                                          d->m_paths.begin(),
                                           [this](const QString &path) {
                                               return is_blacklisted(path, d->m_blacklist);
-                                          }).second, d->m_paths.end());
+                                          })
+                         .second,
+                     d->m_paths.end());
     if (d->m_watcher && !blacklisted.empty()) {
         d->m_watcher->removePaths(blacklisted);
     }
@@ -261,10 +279,11 @@ static QStringList resolve(const QStringList &paths, const QStringList &blacklis
         return QStringList();
     }
     QStringList result;
-    for (const QString &path : paths)
+    for (const QString &path : paths) {
         if (QDir(path).exists()) {
             result += list_dir_absolute(path, blacklist, whitelist);
         }
+    }
     return result + resolve(result, blacklist, whitelist);
 }
 
