@@ -7,34 +7,45 @@
     SPDX-FileCopyrightText: 2016 Bundesamt für Sicherheit in der Informationstechnik
     SPDX-FileContributor: Intevation GmbH
 
-    SPDX-FileCopyrightText: 2020 g10 Code GmbH
+    SPDX-FileCopyrightText: 2020-2022 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "gnupg.h"
 
-#include "utils/compat.h"
-#include "utils/cryptoconfig.h"
-#include "utils/hex.h"
+#include "assuan.h"
+#include "compat.h"
+#include "compliance.h"
+#include "cryptoconfig.h"
+#include "hex.h"
+
+#include <libkleo_debug.h>
+
+#include <KLocalizedString>
+
+#include <QGpgME/CryptoConfig>
+#include <QGpgME/Protocol>
+
+#include <QByteArray>
+#include <QCoreApplication>
+#include <QDateTime>
+#include <QDir>
+#include <QFile>
+#include <QPointer>
+#include <QProcess>
+#include <QRegExp>
+#include <QRegularExpression>
+#include <QStandardPaths>
+#include <QString>
 
 #include <gpgme++/engineinfo.h>
 #include <gpgme++/error.h>
 #include <gpgme++/key.h>
 
-#include <QGpgME/Protocol>
-#include <QGpgME/CryptoConfig>
-
-#include "libkleo_debug.h"
-
-#include <QDir>
-#include <QFile>
-#include <QString>
-#include <QProcess>
-#include <QByteArray>
-#include <QStandardPaths>
-#include <QCoreApplication>
-#include <QRegularExpression>
 #include <gpg-error.h>
 
 #ifdef Q_OS_WIN
@@ -44,13 +55,11 @@
 #include <algorithm>
 #include <array>
 
-#include <KLocalizedString>
-
 using namespace GpgME;
 
 QString Kleo::gnupgHomeDirectory()
 {
-    static QString homeDir = QString::fromUtf8(GpgME::dirInfo("homedir"));
+    static const QString homeDir = QString::fromUtf8(GpgME::dirInfo("homedir"));
     return homeDir;
 }
 
@@ -85,29 +94,40 @@ QString Kleo::gpgPath()
 
 QStringList Kleo::gnupgFileWhitelist()
 {
-    return QStringList()
-           // The obvious pubring
-           << QStringLiteral("pubring.gpg")
-           // GnuPG 2.1 pubring
-           << QStringLiteral("pubring.kbx")
-           // Trust in X509 Certificates
-           << QStringLiteral("trustlist.txt")
-           // Trustdb controls ownertrust and thus WOT validity
-           << QStringLiteral("trustdb.gpg")
-           // We want to update when smartcard status changes
-           << QStringLiteral("reader*.status")
-           // No longer used in 2.1 but for 2.0 we want this
-           << QStringLiteral("secring.gpg")
-           // Changes to the trustmodel / compliance mode might
-           // affect validity so we check this, too.
-           // Globbing for gpg.conf* here will trigger too often
-           // as gpgconf creates files like gpg.conf.bak or
-           // gpg.conf.tmp12312.gpgconf that should not trigger
-           // a change.
-           << QStringLiteral("gpg.conf")
-           << QStringLiteral("gpg.conf-?")
-           << QStringLiteral("gpg.conf-?.?")
-           ;
+    return {
+        // The obvious pubring
+        QStringLiteral("pubring.gpg"),
+        // GnuPG 2.1 pubring
+        QStringLiteral("pubring.kbx"),
+        // Trust in X509 Certificates
+        QStringLiteral("trustlist.txt"),
+        // Trustdb controls ownertrust and thus WOT validity
+        QStringLiteral("trustdb.gpg"),
+        // We want to update when smartcard status changes
+        QStringLiteral("reader*.status"),
+        // No longer used in 2.1 but for 2.0 we want this
+        QStringLiteral("secring.gpg"),
+        // Secret keys (living under private-keys-v1.d/)
+        QStringLiteral("*.key"),
+        // Changes to the trustmodel / compliance mode might
+        // affect validity so we check this, too.
+        // Globbing for gpg.conf* here will trigger too often
+        // as gpgconf creates files like gpg.conf.bak or
+        // gpg.conf.tmp12312.gpgconf that should not trigger
+        // a change.
+        QStringLiteral("gpg.conf"),
+        QStringLiteral("gpg.conf-?"),
+        QStringLiteral("gpg.conf-?.?"),
+    };
+}
+
+QStringList Kleo::gnupgFolderWhitelist()
+{
+    static const QDir gnupgHome{gnupgHomeDirectory()};
+    return {
+        gnupgHome.path(),
+        gnupgHome.filePath(QStringLiteral("private-keys-v1.d")),
+    };
 }
 
 namespace
@@ -129,33 +149,32 @@ private:
     QString mVersion;
     QString mDescription;
     QString mDescLong;
+    QString mBrandingWindowTitle;
+    QString mBrandingIcon;
     bool mSignedVersion;
 
-    Gpg4win() :
-        mVersion(QStringLiteral("Unknown Windows Version")),
-        mDescription(i18n("Certificate Manager and Unified Crypto GUI")),
-        mDescLong(QStringLiteral("<a href=https://www.gpg4win.org>Visit the Gpg4win homepage</a>")),
-        mSignedVersion(false)
+    Gpg4win()
+        : mVersion(QStringLiteral("Unknown Windows Version"))
+        , mDescription(i18n("Certificate Manager and Unified Crypto GUI"))
+        , mDescLong(QStringLiteral("<a href=https://www.gpg4win.org>Visit the Gpg4win homepage</a>"))
+        , mSignedVersion(false)
     {
         const QString instPath = Kleo::gpg4winInstallPath();
         const QString verPath = instPath + QStringLiteral("/../VERSION");
         QFile versionFile(verPath);
 
-        QString versVersion;
-        QString versDescription;
-        QString versDescLong;
         // Open the file first to avoid a verify and then read issue where
         // "auditors" might say its an issue,...
         if (!versionFile.open(QIODevice::ReadOnly)) {
-            // No need to translate this should only be the case in development
-            // builds.
             return;
-        } else {
-            // Expect a three line format of three HTML strings.
-            versVersion = QString::fromUtf8(versionFile.readLine()).trimmed();
-            versDescription = QString::fromUtf8(versionFile.readLine()).trimmed();
-            versDescLong = QString::fromUtf8(versionFile.readLine()).trimmed();
         }
+        // Expect a three line format of three HTML strings.
+        const auto versVersion = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto versDescription = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto versDescLong = QString::fromUtf8(versionFile.readLine()).trimmed();
+        // read optional two branding strings
+        const auto brandingWindowTitle = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto brandingIcon = QString::fromUtf8(versionFile.readLine()).trimmed();
 
         const QString sigPath = verPath + QStringLiteral(".sig");
         QFileInfo versionSig(instPath + QStringLiteral("/../VERSION.sig"));
@@ -165,19 +184,16 @@ private:
             QProcess gpgv;
             gpgv.setProgram(Kleo::gpgPath().replace(QStringLiteral("gpg.exe"), QStringLiteral("gpgv.exe")));
             const QString keyringPath(QStringLiteral("%1/../share/gnupg/distsigkey.gpg").arg(Kleo::gnupgInstallPath()));
-            gpgv.setArguments(QStringList() << QStringLiteral("--keyring")
-                                            << keyringPath
-                                            << QStringLiteral("--")
-                                            << sigPath
-                                            << verPath);
+            gpgv.setArguments(QStringList() << QStringLiteral("--keyring") << keyringPath << QStringLiteral("--") << sigPath << verPath);
             gpgv.start();
             gpgv.waitForFinished();
-            if (gpgv.exitStatus() == QProcess::NormalExit &&
-                !gpgv.exitCode()) {
+            if (gpgv.exitStatus() == QProcess::NormalExit && !gpgv.exitCode()) {
                 qCDebug(LIBKLEO_LOG) << "Valid Version: " << versVersion;
                 mVersion = versVersion;
                 mDescription = versDescription;
                 mDescLong = versDescLong;
+                mBrandingWindowTitle = brandingWindowTitle;
+                mBrandingIcon = brandingIcon;
                 mSignedVersion = true;
             } else {
                 qCDebug(LIBKLEO_LOG) << "gpgv failed with stderr: " << gpgv.readAllStandardError();
@@ -189,6 +205,7 @@ private:
         // Also take Version information from unsigned Versions.
         mVersion = versVersion;
     }
+
 public:
     const QString &version() const
     {
@@ -206,6 +223,14 @@ public:
     {
         return mSignedVersion;
     }
+    const QString &brandingWindowTitle() const
+    {
+        return mBrandingWindowTitle;
+    }
+    const QString &brandingIcon() const
+    {
+        return mBrandingIcon;
+    }
 };
 } // namespace
 
@@ -220,8 +245,7 @@ QString Kleo::gpg4winVersionNumber()
     // we assume that Gpg4win::version() returns a version number (conforming to the semantic
     // versioning spec) optionally prefixed with some text followed by a dash,
     // e.g. "Gpg4win-3.1.15-beta15"; see https://dev.gnupg.org/T5663
-    static const QRegularExpression catchSemVerRegExp{
-        QLatin1String{R"(-([0-9]+(?:\.[0-9]+)*(?:-[.0-9A-Za-z-]+)?(?:\+[.0-9a-zA-Z-]+)?)$)"}};
+    static const QRegularExpression catchSemVerRegExp{QLatin1String{R"(-([0-9]+(?:\.[0-9]+)*(?:-[.0-9A-Za-z-]+)?(?:\+[.0-9a-zA-Z-]+)?)$)"}};
 
     QString ret;
     const auto match = catchSemVerRegExp.match(gpg4winVersion());
@@ -247,20 +271,25 @@ QString Kleo::gpg4winLongDescription()
     return Gpg4win::instance()->longDescription();
 }
 
+QString Kleo::brandingWindowTitle()
+{
+    return Gpg4win::instance()->brandingWindowTitle();
+}
+QString Kleo::brandingIcon()
+{
+    return Gpg4win::instance()->brandingIcon();
+}
+
 QString Kleo::gpg4winInstallPath()
 {
 #ifdef Q_OS_WIN
     // QApplication::applicationDirPath is only used as a fallback
     // to support the case where Kleopatra is not installed from
     // Gpg4win but Gpg4win is also installed.
-    char *instDir = read_w32_registry_string("HKEY_LOCAL_MACHINE",
-                                             "Software\\GPG4Win",
-                                             "Install Directory");
+    char *instDir = read_w32_registry_string("HKEY_LOCAL_MACHINE", "Software\\GPG4Win", "Install Directory");
     if (!instDir) {
         // Fallback to HKCU
-        instDir = read_w32_registry_string("HKEY_CURRENT_USER",
-                                           "Software\\GPG4Win",
-                                           "Install Directory");
+        instDir = read_w32_registry_string("HKEY_CURRENT_USER", "Software\\GPG4Win", "Install Directory");
     }
     if (instDir) {
         QString ret = QString::fromLocal8Bit(instDir) + QStringLiteral("/bin");
@@ -274,19 +303,14 @@ QString Kleo::gpg4winInstallPath()
 
 QString Kleo::gnupgInstallPath()
 {
-
 #ifdef Q_OS_WIN
     // QApplication::applicationDirPath is only used as a fallback
     // to support the case where Kleopatra is not installed from
     // Gpg4win but Gpg4win is also installed.
-    char *instDir = read_w32_registry_string("HKEY_LOCAL_MACHINE",
-                                             "Software\\GnuPG",
-                                             "Install Directory");
+    char *instDir = read_w32_registry_string("HKEY_LOCAL_MACHINE", "Software\\GnuPG", "Install Directory");
     if (!instDir) {
         // Fallback to HKCU
-        instDir = read_w32_registry_string("HKEY_CURRENT_USER",
-                                           "Software\\GnuPG",
-                                           "Install Directory");
+        instDir = read_w32_registry_string("HKEY_CURRENT_USER", "Software\\GnuPG", "Install Directory");
     }
     if (instDir) {
         QString ret = QString::fromLocal8Bit(instDir) + QStringLiteral("/bin");
@@ -324,13 +348,13 @@ QString Kleo::gpgConfListDir(const char *which)
                 --end;
             }
             const QString result = QDir::fromNativeSeparators(QFile::decodeName(hexdecode(line.mid(begin, end - begin))));
-            qCDebug(LIBKLEO_LOG) << "gpgConfListDir: found " << qPrintable(result)
-                                   << " for '" << which << "'entry";
+            qCDebug(LIBKLEO_LOG) << "gpgConfListDir: found " << qPrintable(result) << " for '" << which << "'entry";
             return result;
         }
     }
     qCDebug(LIBKLEO_LOG) << "gpgConfListDir(): didn't find '" << which << "'"
-                           << "entry in output:\n" << gpgConf.readAllStandardError().constData();
+                         << "entry in output:\n"
+                         << gpgConf.readAllStandardError().constData();
     return QString();
 }
 
@@ -387,14 +411,12 @@ bool Kleo::versionIsAtLeast(const char *minimum, const char *actual)
         return false;
     }
 
-    return !std::lexicographical_compare(std::begin(actual_version), std::end(actual_version),
-                                         std::begin(minimum_version), std::end(minimum_version));
-
+    return !std::lexicographical_compare(std::begin(actual_version), std::end(actual_version), std::begin(minimum_version), std::end(minimum_version));
 }
 
 bool Kleo::engineIsVersion(int major, int minor, int patch, GpgME::Engine engine)
 {
-    static QMap<Engine, std::array<int, 3> > cachedVersions;
+    static QMap<Engine, std::array<int, 3>> cachedVersions;
     const int required_version[] = {major, minor, patch};
     // Gpgconf means spawning processes which is expensive on windows.
     std::array<int, 3> actual_version;
@@ -409,10 +431,7 @@ bool Kleo::engineIsVersion(int major, int minor, int patch, GpgME::Engine engine
         bool ok;
         actual_version = getVersionFromString(actual, ok);
 
-        qCDebug(LIBKLEO_LOG) << "Parsed" << actual << "as: "
-                               << actual_version[0] << '.'
-                               << actual_version[1] << '.'
-                               << actual_version[2] << '.';
+        qCDebug(LIBKLEO_LOG) << "Parsed" << actual << "as: " << actual_version[0] << '.' << actual_version[1] << '.' << actual_version[2] << '.';
         if (!ok) {
             return false;
         }
@@ -422,15 +441,14 @@ bool Kleo::engineIsVersion(int major, int minor, int patch, GpgME::Engine engine
     }
 
     // return ! ( actual_version < required_version )
-    return !std::lexicographical_compare(std::begin(actual_version), std::end(actual_version),
-                                         std::begin(required_version), std::end(required_version));
+    return !std::lexicographical_compare(std::begin(actual_version), std::end(actual_version), std::begin(required_version), std::end(required_version));
 }
 
-const QString& Kleo::paperKeyInstallPath()
+const QString &Kleo::paperKeyInstallPath()
 {
-    static const QString pkPath = QStandardPaths::findExecutable(QStringLiteral("paperkey"), QStringList() << QCoreApplication::applicationDirPath()).isEmpty() ?
-                                  QStandardPaths::findExecutable(QStringLiteral("paperkey")) :
-                                  QStandardPaths::findExecutable(QStringLiteral("paperkey"), QStringList() << QCoreApplication::applicationDirPath());
+    static const QString pkPath = (QStandardPaths::findExecutable(QStringLiteral("paperkey"), QStringList() << QCoreApplication::applicationDirPath()).isEmpty()
+                                       ? QStandardPaths::findExecutable(QStringLiteral("paperkey"))
+                                       : QStandardPaths::findExecutable(QStringLiteral("paperkey"), QStringList() << QCoreApplication::applicationDirPath()));
     return pkPath;
 }
 
@@ -454,8 +472,8 @@ QString Kleo::keyserver()
 
 bool Kleo::haveX509DirectoryServerConfigured()
 {
-    return !getCryptoConfigUrlList("dirmngr", "ldapserver").empty()
-        || !getCryptoConfigUrlList("dirmngr", "LDAP Server").empty()
+    return !getCryptoConfigUrlList("dirmngr", "ldapserver").empty() //
+        || !getCryptoConfigUrlList("dirmngr", "LDAP Server").empty() //
         || !getCryptoConfigUrlList("gpgsm", "keyserver").empty();
 }
 
@@ -468,27 +486,20 @@ bool Kleo::gpgComplianceP(const char *mode)
 
 bool Kleo::gnupgUsesDeVsCompliance()
 {
-    return getCryptoConfigStringValue("gpg", "compliance") == QLatin1String{"de-vs"};
+    return DeVSCompliance::isActive();
 }
 
 bool Kleo::gnupgIsDeVsCompliant()
 {
-    if (!gnupgUsesDeVsCompliance()) {
-        return false;
-    }
-    // the pseudo option compliance_de_vs was added in 2.2.28; for older versions of GnuPG
-    // we assume non-compliance to be on the safe side; versions of Kleopatra for which
-    // this matters are bundled with new enough versions of GnuPG anyway
-    return getCryptoConfigIntValue("gpg", "compliance_de_vs", 0) != 0;
+    return DeVSCompliance::isCompliant();
 }
 
 enum GpgME::UserID::Validity Kleo::keyValidity(const GpgME::Key &key)
 {
     enum UserID::Validity validity = UserID::Validity::Unknown;
 
-    for (const auto &uid: key.userIDs()) {
-        if (validity == UserID::Validity::Unknown
-            || validity > uid.validity()) {
+    for (const auto &uid : key.userIDs()) {
+        if (validity == UserID::Validity::Unknown || validity > uid.validity()) {
             validity = uid.validity();
         }
     }
@@ -497,14 +508,14 @@ enum GpgME::UserID::Validity Kleo::keyValidity(const GpgME::Key &key)
 }
 
 #ifdef Q_OS_WIN
-static QString fromEncoding (unsigned int src_encoding, const char *data)
+static QString fromEncoding(unsigned int src_encoding, const char *data)
 {
     int n = MultiByteToWideChar(src_encoding, 0, data, -1, NULL, 0);
     if (n < 0) {
         return QString();
     }
 
-    wchar_t *result = (wchar_t *) malloc ((n+1) * sizeof *result);
+    wchar_t *result = (wchar_t *)malloc((n + 1) * sizeof *result);
 
     n = MultiByteToWideChar(src_encoding, 0, data, -1, result, n);
     if (n < 0) {
@@ -529,7 +540,7 @@ QString Kleo::stringFromGpgOutput(const QByteArray &ba)
      * what GetConsoleOutputCP returns for a console application
      * it appears to be the OEMCP.
      */
-    unsigned int cpno = GetConsoleOutputCP ();
+    unsigned int cpno = GetConsoleOutputCP();
     if (!cpno) {
         cpno = GetOEMCP();
     }
@@ -575,3 +586,109 @@ QStringList Kleo::backendVersionInfo()
     }
     return versions;
 }
+
+namespace
+{
+
+template<typename Function1, typename Function2>
+auto startGpgConf(const QStringList &arguments, Function1 onSuccess, Function2 onFailure)
+{
+    auto process = new QProcess;
+    process->setProgram(Kleo::gpgConfPath());
+    process->setArguments(arguments);
+
+    QObject::connect(process, &QProcess::started, [process]() {
+        qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") was started successfully";
+    });
+    QObject::connect(process, &QProcess::errorOccurred, [process, onFailure](auto error) {
+        qCDebug(LIBKLEO_LOG).nospace() << "Error while running gpgconf (" << process << "): " << error;
+        process->deleteLater();
+        onFailure();
+    });
+    QObject::connect(process, &QProcess::readyReadStandardError, [process]() {
+        for (const auto &line : process->readAllStandardError().trimmed().split('\n')) {
+            qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") stderr: " << line;
+        }
+    });
+    QObject::connect(process, &QProcess::readyReadStandardOutput, [process]() {
+        (void)process->readAllStandardOutput(); /* ignore stdout */
+    });
+    QObject::connect(process,
+                     qOverload<int, QProcess::ExitStatus>(&QProcess::finished),
+                     [process, onSuccess, onFailure](int exitCode, QProcess::ExitStatus exitStatus) {
+                         if (exitStatus == QProcess::NormalExit) {
+                             qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") exited (exit code: " << exitCode << ")";
+                             if (exitCode == 0) {
+                                 onSuccess();
+                             } else {
+                                 onFailure();
+                             }
+                         } else {
+                             qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") crashed (exit code: " << exitCode << ")";
+                             onFailure();
+                         }
+                         process->deleteLater();
+                     });
+
+    qCDebug(LIBKLEO_LOG).nospace() << "Starting gpgconf (" << process << ") with arguments " << process->arguments().join(QLatin1Char(' ')) << " ...";
+    process->start();
+
+    return process;
+}
+
+static auto startGpgConf(const QStringList &arguments)
+{
+    return startGpgConf(
+        arguments,
+        []() {},
+        []() {});
+}
+
+}
+
+void Kleo::launchGpgAgent()
+{
+    static QPointer<QProcess> process;
+    static qint64 mSecsSinceEpochOfLastLaunch = 0;
+    static int numberOfFailedLaunches = 0;
+
+    if (Kleo::Assuan::agentIsRunning()) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": gpg-agent is already running";
+        return;
+    }
+    if (process) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": gpg-agent is already being launched";
+        return;
+    }
+    const auto now = QDateTime::currentMSecsSinceEpoch();
+    if (now - mSecsSinceEpochOfLastLaunch < 1000) {
+        // reduce attempts to launch the agent to 1 attempt per second
+        return;
+    }
+    mSecsSinceEpochOfLastLaunch = now;
+    if (numberOfFailedLaunches > 5) {
+        qCWarning(LIBKLEO_LOG) << __func__ << ": Launching gpg-agent failed" << numberOfFailedLaunches << "times in a row. Giving up.";
+        return;
+    }
+
+    process = startGpgConf(
+        {QStringLiteral("--launch"), QStringLiteral("gpg-agent")},
+        []() {
+            numberOfFailedLaunches = 0;
+        },
+        []() {
+            numberOfFailedLaunches++;
+        });
+}
+
+void Kleo::killDaemons()
+{
+    static QPointer<QProcess> process;
+
+    if (process) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": The daemons are already being shut down";
+        return;
+    }
+
+    process = startGpgConf({QStringLiteral("--kill"), QStringLiteral("all")});
+}
